/*-------------------------------------------------------------------------+
| Copyright 2011 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.dnd.gef;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.Request;
import org.eclipse.gef.dnd.AbstractTransferDropTargetListener;
import org.eclipse.swt.dnd.DND;
import org.fortiss.tooling.base.ui.editpart.policy.RelayCreateCommandEditPolicyBase;
import org.fortiss.tooling.base.ui.editpart.request.CreateElementCompositionRequest;
import org.fortiss.tooling.kernel.service.IElementCompositorService;
import org.fortiss.tooling.kernel.ui.dnd.CompositionServiceLocalDNDTransfer;
import org.fortiss.tooling.kernel.ui.dnd.ElementCompositionSource;

/**
 * This class implements a drop target listener for {@link EditPartViewer}s,
 * which creates {@link CreateElementCompositionRequest}s. The request contains
 * an {@link ElementCompositionSource}. The request is usually handled by the
 * target edit part's {@link RelayCreateCommandEditPolicyBase}, which completes
 * the DND operation with the target information. It then relays the completed
 * DND operation to the {@link IElementCompositorService}.
 * <p>
 * This class mimics the behavior of the kernel's SWT-based
 * {@link ElementCompositionDropTargetAdapter}, but uses the GEF abstraction
 * {@link AbstractTransferDropTargetListener} super class.
 * 
 * @see RelayCreateCommandEditPolicyBase
 * @see ElementCompositionDropTargetAdapter
 * 
 * @author hoelzl
 */
public class ElementCompositionDropTargetListener extends AbstractTransferDropTargetListener {

	/** Constructor. */
	public ElementCompositionDropTargetListener(EditPartViewer viewer) {
		super(viewer, CompositionServiceLocalDNDTransfer.getInstance());
		setEnablementDeterminedByCommand(true);
	}

	/** {@inheritDoc} */
	@Override
	protected Request createTargetRequest() {
		return new CreateElementCompositionRequest();
	}

	/** {@inheritDoc} */
	@Override
	protected void updateTargetRequest() {
		CreateElementCompositionRequest req = (CreateElementCompositionRequest)getTargetRequest();
		req.setLocation(getDropLocation());

		ElementCompositionSource sourceData = null;
		if(getCurrentEvent().data instanceof ElementCompositionSource) {
			sourceData = (ElementCompositionSource)getCurrentEvent().data;
		} else if(CompositionServiceLocalDNDTransfer.getInstance().getObject() instanceof ElementCompositionSource) {
			// there have been reports about SWT platform inconsistencies:
			// on OSX getCurrentEvent().data might be null during dragging
			// while it is not null on other platforms. Therefore, we use this
			// fallback.
			sourceData =
					(ElementCompositionSource)CompositionServiceLocalDNDTransfer.getInstance()
							.getObject();
		}
		req.setDragSource(sourceData);
	}

	/** {@inheritDoc} */
	@Override
	protected void handleDragOver() {
		getCurrentEvent().detail = DND.DROP_COPY;
		super.handleDragOver();
	}
}
